/**
	Program to apply a filter in the frequency domain
	using the FFT functions 

	Fraga. Jun 12, 2001
**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "tiff.h"
#include "fft.h"

int do_work( char *namein )
{
	MATRIX *IN;
	MATRIX *FFTIMG;
	FILEINFO finfo;
	int i, j, signo;
	byte *iptr;
	float *fptr;
	double totenergy, val;
	
	

	IN = read_tiff( &finfo, namein ); 
	if ( IN==NULL ) {
		fprintf( stderr, "ERROR reading %s file\n", namein );
		return -1;
	}
	FFTIMG = matrix_allocate( IN->rows, IN->cols + 2, sizeof(float) );

	/** We copy the readed image to the new float matrix **/
	/** The variable signo is for move the FT a half of a period **/
	signo = -1;
	for ( i=0; i<IN->rows; i++ ) {
		iptr = (byte *)IN->ptr[i];
		fptr = (float *)FFTIMG->ptr[i];
		for ( j=0; j<IN->cols; j++ ) {
			signo *= -1;
			*fptr++ = *iptr++ * signo;
	//		*fptr++ = *iptr++;
		}
		signo *= -1;
	}

	/** Now we calculate the FFT **/
	image_fft ( (float **)FFTIMG->ptr, IN->rows , IN->cols, DIRECT );

	/** We measure the total image energy **/
	totenergy = total_energy( (float **)FFTIMG->ptr, IN->rows, IN->cols);
	
	printf( "# Total energy = %e\n", totenergy );
	printf( "# radius  energy  %%energy\n" );
	for ( i=1; i<IN->rows/2; i++ ) {
		val = r_energy( (float **)FFTIMG->ptr, IN->rows, IN->cols, i );
		printf ( "%d %e %3.2f\n", i, val, 100*(val/totenergy) );
	}
	printf ( "%d %e 100.00\n", i+5, totenergy );


	matrix_free( IN );
	matrix_free( FFTIMG );

	return 0;
}

int main( int argc, char *argv[] )
{
	char namein[32];

	if ( argc < 2 ) { 
		fprintf( stderr, "Use: %s image.tif\n", argv[0] );
		return -1;
	}

	strncpy( namein, argv[1], 32 );

	if ( do_work( namein ) >= 0 ) {
		fprintf( stderr, "OK!!!\n" );
		return 0;
	}
	else {
		fprintf( stderr, "ERROR: Something is wrong!!!\n" );
		exit(1);
	}
	return 0;
}
