/**
	Program to produce an equalized image 

	(using the histogram equalization program)
	Input image must be in TIFF format

	Fraga. Jun 4, 2001
**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "tiff.h"
#include "utils.h"

int do_work( char *namein ) 
{
	MATRIX *IN, *EQ;
	FILEINFO finfo;
	int i, j;
	byte *iptr;
	

	IN = read_tiff( &finfo, namein ); 
	if ( IN==NULL ) {
		fprintf( stderr, "ERROR reading %s file\n", namein );
		return -1;
	}
	if ( IN->type != 5 ) {
		fprintf( stderr, "ERROR Bad format. Image must be in 256 grey tones\n");
		return -1;
	}

	EQ = histogram_equalization( IN, 255 );
	if ( EQ == NULL ) {
		fprintf( stderr, "ERROR doing histogram equalization\n");
	 	return -1;
	}
	EQ->type=5;
	if ( write_tiff( EQ, "equalized.tif" ) != GOOD_WRITE )  
        return -1;

	matrix_free( IN );
	matrix_free( EQ );

	return 0;
}

int main( int argc, char *argv[] )
{
	char namein[32];

	if ( argc < 2 ) { 
		fprintf( stderr, "Use: %s image.tif\n", argv[0] );
		return -1;
	}

	strncpy( namein, argv[1], 32 );

	if ( do_work( namein ) >= 0 ) {
		fprintf( stderr, "OK!!!\n" );
		return 0;
	}
	else {
		fprintf( stderr, "ERROR: Something is wrong!!!\n" );
		exit(1);
	}
	return 0;
}
