#include "matrix.h"

void erosion_3x3( MATRIX *IN, MATRIX *OUT )
{
	int i, j, l, k;
	byte *ptrin, *ptrout;
	int val;

	for( i=0; i<IN->rows; i++ ) {
		ptrout = (byte *) OUT->ptr[i];
		for( j=0; j<IN->cols; j++ ) *ptrout++ = 0;
	}

	for( i=0; i<=IN->rows - 3; i++ ) {
		ptrout = (byte *) OUT->ptr[i+1] + 1;
		for( j=0; j<=IN->cols - 3; j++ ) {
			val = 0xff;
			for( l=0; l<3; l++ ) { 
				ptrin = (byte *) IN->ptr[i + l] + j;
				for( k=0; k<3; k++ ) val &= *ptrin++; 
			}
			if ( val ) *ptrout = 255;	
			ptrout++;
		}
	}
}

void dilation_3x3( MATRIX *IN, MATRIX *OUT )
{
	int i, j, l, k;
	byte *ptrin, *ptrout;
	int val;

	for( i=0; i<IN->rows; i++ ) {
		ptrout = (byte *) OUT->ptr[i];
		for( j=0; j<IN->cols; j++ ) *ptrout++ = 0;
	}

	for( i=0; i<=IN->rows - 3; i++ ) {
		ptrout = (byte *) OUT->ptr[i+1] + 1;
		for( j=0; j<=IN->cols - 3; j++ ) {
			val = 0;
			for( l=0; l<3; l++ ) { 
				ptrin = (byte *) IN->ptr[i + l] + j;
				for( k=0; k<3; k++ ) val |= *ptrin++; 
			}
			if ( val ) *ptrout = 255;	
			ptrout++;
		}
	}
}

void opening_3x3( MATRIX *IN, MATRIX *OUT )
{
	int i, j;
	byte *ptrin, *ptrout;

	erosion_3x3( IN, OUT );
	dilation_3x3( OUT, IN );

	/** We copy image IN to OUT **/
	for( i=0; i<IN->rows; i++ ) {
		ptrin = (byte *) IN->ptr[i];
		ptrout = (byte *) OUT->ptr[i];
		for( j=0; j<IN->cols; j++ ) *ptrout++ = *ptrin++;
	}
	/** And the resulted image is finally in OUT **/
}

void closing_3x3( MATRIX *IN, MATRIX *OUT )
{
	int i, j;
	byte *ptrin, *ptrout;

	dilation_3x3( IN, OUT );
	erosion_3x3( OUT, IN );

	/** We copy image IN to OUT **/
	for( i=0; i<IN->rows; i++ ) {
		ptrin = (byte *) IN->ptr[i];
		ptrout = (byte *) OUT->ptr[i];
		for( j=0; j<IN->cols; j++ ) *ptrout++ = *ptrin++;
	}
	/** And the resulted image is finally in OUT **/
}
