/**
	Program to test the morphological operations.

	Produce an output image in PBM format

	Fraga. May 28, 2002
**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "pnm.h"

int do_work( char *namein, char *nameout, int operation ) 
{
	MATRIX *IN, *OUT;

	IN = read_pnm( namein ); 
	if ( IN==NULL ) {
		fprintf( stderr, "ERROR reading %s file\n", namein );
		return -1;
	}
	if ( IN->type != 5 ) {
		fprintf( stderr, "ERROR Bad format. Image must be in b/w\n");
		return -1;
	}
	OUT = matrix_allocate( IN->rows, IN->cols, 1 );
	if ( OUT == NULL ) {
		fprintf( stderr, "ERROR allocating output image\n");
		return -1;
	}
	switch( operation ) {
		case 0:
			erosion_3x3( IN, OUT );
			break;
		case 1:
			dilation_3x3( IN, OUT );
			break;
		case 2:
			opening_3x3( IN, OUT );
			break;
		case 3:
			closing_3x3( IN, OUT );
			break;

	}

	OUT->type = 5;
	if ( write_pnm( OUT, nameout ) != 0 )  
        return -1;

	matrix_free( OUT );
	matrix_free( IN );

	return 0;
}

int main( int argc, char *argv[] )
{
	char namein[64];
	char nameout[64];
	int ope;

	if ( argc < 4 ) { 
		fprintf( stderr, "Use: %s input_image.pbm output_image.pbm operation\n",
			argv[0]);
		fprintf( stderr, "\twhere operation is:\n" );
		fprintf( stderr, "\t  0 erosion\n" );
		fprintf( stderr, "\t  1 dilation\n" );
		fprintf( stderr, "\t  2 opening\n" );
		fprintf( stderr, "\t  3 closing\n" );
		return -1;
	}

	strncpy( namein, argv[1], 64 );
	strncpy( nameout, argv[2], 64 );
	ope = atoi( argv[3] );

	if ( do_work( namein, nameout, ope ) >= 0 ) {
		fprintf( stderr, "OK!!!\n" );
		return 0;
	}
	else {
		fprintf( stderr, "ERROR: Something is wrong!!!\n" );
		exit(1);
	}
	return 0;
}
